#!/usr/bin/env python3 
import json,pickle,math,matplotlib,sys,os,string,subprocess
import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
from plateypus.dataprocessing.miscFunctions import cleanUpFlowjoCSV,reorderDfByInputOrder
from plateypus.dataprocessing import cytokineDataProcessing,singleCellDataProcessing,cellDataProcessing

dataTypeLevelNames = {'cyt':['Cytokine'],'cell':['CellType','Marker','Statistic'],'prolif':['Statistic'],'singlecell':['CellType']}
dataTypeDataFrameFileNames = {'cyt':'cytokineConcentrationPickleFile','cell':'cellStatisticPickleFile','prolif':'proliferationStatisticPickleFile','singlecell':'initialSingleCellPickleFile','killing':'killingIndexPickleFile'}
plateRowLetters = list(string.ascii_uppercase)[:16]
plateColumnNumbers = list(range(1,25))

def returnMultiIndex(sortedData,sortedFiles,dataType,folderName):
    if(dataType == 'cyt'):
        newMultiIndex = cytokineDataProcessing.parseCytokineCSVHeaders(pd.read_csv('inputData/bulkCSVFiles/A1_'+dataType+'.csv').columns)
    elif(dataType == 'cell'):
        if 'antibodyPanel-'+folderName+'.csv' in os.listdir('misc'):
            panelData = pd.read_csv('misc/antibodyPanel-'+folderName+'.csv',)
            newMultiIndex = cellDataProcessing.parseCellCSVHeaders(pd.read_csv('inputData/bulkCSVFiles/A1_'+dataType+'.csv').columns,panelData=panelData)
        else:
            newMultiIndex = cellDataProcessing.parseCellCSVHeaders(pd.read_csv('inputData/bulkCSVFiles/A1_'+dataType+'.csv').columns)
    elif(dataType == 'cytcorr'):
        newMultiIndex = []
    if dataType != 'singlecell':
        return sortedData,newMultiIndex
    else:
        return sortedFiles,newMultiIndex

def decodeBarcodedPlates(experimentParameters,folderName,dataType,reverse):
    path = 'inputData/bulkCSVFiles/'
    barcodingDict = experimentParameters['barcodingDict']
    bIndex  = 0
    alignmentColumnOrder = []
    for barcodedPlate in barcodingDict:
        barcodedCSV = pd.read_csv(path+barcodedPlate+'_'+dataType+'.csv')
        for decodedPlate in barcodingDict[barcodedPlate]:
            decodingIndices = [0]
            barcodes = barcodingDict[barcodedPlate][decodedPlate]
            for col,columnHeader in enumerate(barcodedCSV.columns):
                includeInDecoding = True
                for barcode in barcodes:
                    if barcode.lower().replace(' ','') not in columnHeader.lower().replace(' ',''):
                        includeInDecoding = False
                if includeInDecoding:
                    decodingIndices.append(col)
            decodingIndices.append(-1)
            decodedCSV = barcodedCSV.iloc[:,decodingIndices]
            newColumns = []
            for col,column in enumerate(decodedCSV.columns):
                if col == 0:
                    newColumns.append('')
                elif col == len(decodedCSV.columns)-1:
                    newColumns.append('')
                else:
                    populationStatisticSplit = column.split(' | ')
                    population = populationStatisticSplit[0]
                    splitPopulations = population.split('/')
                    populationsToKeep = []
                    for i,splitPopulation in enumerate(splitPopulations):
                        keepPopulation = True
                        for barcode in barcodes:
                            if barcode.lower().replace(' ','') in splitPopulation.lower().replace(' ',''):
                                keepPopulation = False
                        if keepPopulation:
                            populationsToKeep.append(splitPopulation)
                    newColumn = ' | '.join(['/'.join(populationsToKeep),populationStatisticSplit[1]])
                    newColumns.append(newColumn)
            decodedCSV.columns = newColumns
            if bIndex == 0:
                alignmentColumnOrder = decodedCSV.columns.tolist()
            else:
                if alignmentColumnOrder != decodedCSV.columns.tolist():
                    #Realign columns
                    newDecodedCSV = decodedCSV.iloc[:,:-1][alignmentColumnOrder[:-1]]
                    decodedCSV.iloc[:,:-1] = newDecodedCSV.values
                    decodedCSV.columns = alignmentColumnOrder
            
            decodedCSV.to_csv(path+decodedPlate+'_'+dataType+'.csv',index=False)
            bIndex+=1
        
def unpackMultiplexedPlates(experimentParameters,folderName,dataType,reverse):
    #A1->A1,A2->A2,A3- >A1,B1->A4,B2->A3
    #1.1,1.3,1.5,3.1,3.3,3.5 -> A1; 1.2,1.4,1.6,3.2,3.4,3.6->A2; 2.1,2.3,3.5,4.1,4.3,4.5->A4; 2.2,2.4,2.6,4.2,4.4,4.6->A3
    #1-24
    #A-P
    wellIDConversionDict = {}
    for i,plateLetter in enumerate(plateRowLetters):
        for j,plateNumber in enumerate(plateColumnNumbers):
            wellIDConversionDict[plateLetter+str(plateNumber)] = plateRowLetters[int(i/2)]+str(plateColumnNumbers[int(j/2)])
    plateIDConversionDict = {}
    for multiplexedPlateName in experimentParameters['unpackingDict']:
        for i,multiplexedWellPos in enumerate(experimentParameters['unpackingDict'][multiplexedPlateName]):
            wellIDs = []
            if multiplexedWellPos != '':
                if i < 2:
                    j = 0
                else:
                    j = 1
                for rowIndex in range(j,len(plateRowLetters),2):
                    for colIndex in range(i%2,len(plateColumnNumbers),2):
                        wellID = plateRowLetters[rowIndex]+str(plateColumnNumbers[colIndex])
                        wellIDs.append(wellID)
            plateIDConversionDict[multiplexedWellPos] = wellIDs
    #Specimen_001_A1_A01_001.fcs
    multiplexedPlateNames = experimentParameters['unpackingDict'].keys()
    #Unpacking
    if not reverse:
        for multiplexedPlateName in multiplexedPlateNames:
            multiplexedWellPoses = experimentParameters['unpackingDict'][multiplexedPlateName]
            with open('inputData/bulkCSVFiles/'+multiplexedPlateName+'_'+dataType+'.csv', 'r') as f:
                multiplexedCSVLines = f.readlines()
            for multiplexedWellPos in multiplexedWellPoses:
                if multiplexedWellPos != '':
                    wellIDsInThisPos = plateIDConversionDict[multiplexedWellPos]
                    linesToMove = []
                    newCSVLines = []
                    for lineNum,line in enumerate(multiplexedCSVLines):
                        if lineNum in [0,len(multiplexedCSVLines)-2,len(multiplexedCSVLines)-1]:
                            newCSVLines.append(line)
                        else:
                            #CyTEK Explorer
                            if ' Well' in line:
                                wellID = line.split(' ')[0]
                            #FACSDiva
                            else:
                                fileName = line.split(',')[0]
                                wellID = fileName.split('_')[2]
                            #Well ID is in pos or first line or last two lines
                            if wellID in wellIDsInThisPos:
                                #underscorePoses = [pos for pos, char in enumerate(line) if char == '_']
                                newWellID = wellIDConversionDict[wellID]
                                newLine = line.replace(wellID,newWellID)
                                newCSVLines.append(newLine)
                    with open('inputData/bulkCSVFiles/'+multiplexedWellPos+'_'+dataType+'.csv', 'w') as f:
                        for item in newCSVLines:
                            f.write("%s" % item)
    #Repacking
    else:
        #Invert well conversion dictionary
        for mpos,multiplexedPlateName in enumerate(experimentParameters['unpackingDict']):
            newCSVLines = []
            for mpos2,multiplexedWellPos in enumerate(experimentParameters['unpackingDict'][multiplexedPlateName]):
                specificWellIDConversionDict = {}
                for well in plateIDConversionDict[multiplexedWellPos]:
                    specificWellIDConversionDict[well] = wellIDConversionDict[well]
                invertedWellIDConversionDict = {v: k for k, v in specificWellIDConversionDict.items()}
                if multiplexedWellPos != '':
                    #Read unpacked csv, rename to name-unpacked.csv, delete old name (as repacked csv names can overlap with these)
                    with open('inputData/bulkCSVFiles/'+multiplexedWellPos+'_'+dataType+'.csv', 'r') as f:
                        multiplexedCSVLines = f.readlines()
                    subprocess.run(['cp','inputData/bulkCSVFiles/'+multiplexedWellPos+'_'+dataType+'.csv','inputData/bulkCSVFiles/'+multiplexedWellPos+'-unpacked_'+dataType+'.csv'])
                    subprocess.run(['rm','inputData/bulkCSVFiles/'+multiplexedWellPos+'_'+dataType+'.csv'])
                    
                    for lineNum,line in enumerate(multiplexedCSVLines):
                        #If first plate to repack, use first line
                        #If last plate to repack, use last 2 lines
                        if lineNum in [0,len(multiplexedCSVLines)-2,len(multiplexedCSVLines)-1]:
                            if (lineNum == 0 and mpos2 == 0) or (lineNum in [len(multiplexedCSVLines)-2,len(multiplexedCSVLines)-1] and mpos2 == len(experimentParameters['unpackingDict'][multiplexedPlateName])-1):
                                newCSVLines.append(line)
                        else:
                            #CyTEK Explorer
                            if ' Well' in line:
                                wellID = line.split(' ')[0]
                            #FACSDiva
                            else:
                                fileName = line.split(',')[0]
                                wellID = fileName.split('_')[2]
                            newWellID = invertedWellIDConversionDict[wellID]        
                            #print(wellID+'->'+newWellID)
                            newLine = line.replace(wellID,newWellID)
                            newCSVLines.append(newLine)
                     
            with open('inputData/bulkCSVFiles/'+multiplexedPlateName+'_'+dataType+'.csv', 'w') as f:
                for item in newCSVLines:
                    f.write("%s" % item)

def performCommaCheck(fileName):
    with open('inputData/bulkCSVFiles/'+fileName, 'r') as istr:
        with open('inputData/bulkCSVFiles/temp-'+fileName, 'w') as ostr:
            for line in istr:
                if line[-1] != ',':
                    line = line.rstrip('\n') + ','
                    print(line, file=ostr)
                else:
                    line = line.rstrip('\n')
                    print(line, file=ostr)
    subprocess.run(['rm','inputData/bulkCSVFiles/'+fileName])
    subprocess.run(['mv','inputData/bulkCSVFiles/temp-'+fileName,'inputData/bulkCSVFiles/'+fileName])

def createBaseDataFrame(experimentParameters,folderName,experimentNumber,dataType,layoutDict):
    if experimentParameters['format'] == 'tube':
        fullFormatDf = pickle.load(open('misc/tubeLayout-'+folderName+'-cell.pkl','rb'))
        dfList = []
        levelLabelDict = experimentParameters['levelLabelDict']
        for fileName in os.listdir('inputData/bulkCSVFiles/'):
            if '.csv' in fileName:
                performCommaCheck(fileName)
                
                bulkTubeCSVFileName = fileName
                columnMultiIndexTuples = cellDataProcessing.parseCellCSVHeaders(pd.read_csv('inputData/bulkCSVFiles/'+bulkTubeCSVFileName).columns)
                columnMultiIndex = pd.MultiIndex.from_tuples(columnMultiIndexTuples,names=['CellType','Marker','Statistic'])

                fullData = pd.read_csv('inputData/bulkCSVFiles/'+bulkTubeCSVFileName,header=0)
                if 'Unnamed' in fullData.columns[-1]:
                    data = fullData.iloc[:-2,1:-1].values
                else:
                    data = fullData.iloc[:-2,1:].values
                sampleNames = fullData.iloc[:-2,0].values.ravel()
                sampleIndexStart = fullFormatDf.values.ravel().tolist().index(sampleNames[0])
                sampleIndexEnd = fullFormatDf.values.ravel().tolist().index(sampleNames[-1])
                rowMultiIndex = fullFormatDf.iloc[sampleIndexStart:sampleIndexEnd+1,:].index
                
                timeDataList = []
                timeSubsets = []
                times = levelLabelDict[list(levelLabelDict.keys())[-1]]

                #Can use sample name file to assign time values
                if 'sampleNameFile.xlsx' in os.listdir('misc') or 'sampleNameFile.csv' in os.listdir('misc'):
                    if 'sampleNameFile.xlsx' in os.listdir('misc'): 
                        sampleNameDf = pd.read_excel('misc/sampleNameFile.xlsx')
                    else:
                        sampleNameDf = pd.read_csv('misc/sampleNameFile.csv')
                    if 'Time' in sampleNameDf.columns:
                        for time in times:
                            timeIndices = []
                            for row in range(sampleNameDf.shape[0]):
                                if sampleNameDf[list(levelLabelDict.keys())[-1]].values[row] == time:
                                    timeIndices.append(row)
                            timeSubsets.append(timeIndices)
                    #Otherwise just assume 1 timepoint (HACK NEED TO FIX EVENTUALLY)
                    else:
                        timeSubsets.append(list(range(data.shape[0])))
                #Otherwise just assume 1 timepoint (HACK NEED TO FIX EVENTUALLY)
                else:
                    timeSubsets.append(list(range(data.shape[0])))

                for timeSubset in timeSubsets: 
                    dataList = []
                    columnTupleList = []
                    for i,columnTuple in enumerate(columnMultiIndexTuples):
                        ser = pd.Series(data[timeSubset,i],index=rowMultiIndex)
                        dataList.append(ser)
                        columnTupleList.append(tuple(columnTuple))
                    fullExperimentDf = pd.concat(dataList,keys=columnTupleList,names=['CellType','Marker','Statistic'])
                    timeDataList.append(fullExperimentDf)
                
                k = pd.concat(timeDataList,keys=times,names=[list(levelLabelDict.keys())[-1]])
                repeatList = []
                for name in k.index:
                    if name not in repeatList:
                        repeatList.append(name)
                    else:
                        print('Repeated:')
                        print(name)
                partialExperimentDf = pd.concat(timeDataList,keys=times,names=[list(levelLabelDict.keys())[-1]]).unstack(list(levelLabelDict.keys())[-1])
                dfList.append(partialExperimentDf)

        fullExperimentDf = pd.concat(dfList)
    else:
        realDataType = dataType
        
        #Reverse order; first decode 96 well barcoded plates, then repack the unbarcoded 96 well plates to a 384 well plate 
        if experimentParameters['overallPlateDimensions'][0] == 16 and 'unpackingDict' in list(experimentParameters.keys()):
            reverse = True
        #Normal order; first decode 384 well barcoded plates, then unpack the unbarcoded 384 well plates to 96 well plates
        else:
            reverse = False
        if 'barcodingDict' in list(experimentParameters.keys()):
            decodeBarcodedPlates(experimentParameters,folderName,dataType,reverse)
        if 'unpackingDict' in list(experimentParameters.keys()):
            unpackMultiplexedPlates(experimentParameters,folderName,dataType,reverse)

        #Legacy experiment parameter files compatibility
        if 'paired' in experimentParameters.keys():
            if experimentParameters['paired']:
                numRowPlates = 2
            else:
                numRowPlates = 1
            numColumnPlates = int(experimentParameters['numPlates']/numRowPlates)
        else:
            numRowPlates = 1 
            numColumnPlates = experimentParameters['numPlates']

        #Combine plate and well IDs into a single ID val for every single sample in the experiment
        identificationMatrix = np.empty(layoutDict['plateID'].shape,dtype=object)
        for row in range(identificationMatrix.shape[0]):
            for col in range(identificationMatrix.shape[1]):
                wellID = layoutDict['wellID'][row,col]
                plateID = layoutDict['plateID'][row,col]
                fullID = plateID+'-'+wellID
                identificationMatrix[row,col] = fullID

        plateNames = np.unique(layoutDict['plateID'])
        
        levelLabelDict = experimentParameters['levelLabelDict']
        plateDimensions = experimentParameters['overallPlateDimensions'] 
        levels = list(levelLabelDict.keys())
        conditionLevels = list(levelLabelDict.keys())[:-1]
        conditionLevelValues = levelLabelDict.copy()
        del conditionLevelValues[list(levelLabelDict.keys())[-1]]
        allLevelValues = experimentParameters['levelLabelDict']
        
        sortedData,sortedFiles = cleanUpFlowjoCSV(plateNames,folderName,dataType,experimentParameters)
        allRawData,newLevelList = returnMultiIndex(sortedData,sortedFiles,realDataType,folderName)

        #print(allRawData)
        dfList = []
        for rawData,plateID in zip(allRawData,plateNames):
            fullTupleList = []
            index = 0
            for row in range(rawData.shape[0]):
                sampleID = int(rawData.iloc[row,0])-1
                wellID = plateRowLetters[int(sampleID/plateDimensions[1])]+str(plateColumnNumbers[sampleID % plateDimensions[1]])
                fullID = plateID+'-'+wellID
                
                sampleLocation = np.argwhere(identificationMatrix == fullID)[0]
                column = []
                tupleList = []
                for levelID in layoutDict['keys']:
                    levelValueID = layoutDict['keys'][levelID][sampleLocation[0],sampleLocation[1]]
                    if levelValueID != 'blank':
                        level = levels[levelID]
                        levelValue = allLevelValues[level][levelValueID]
                        tupleList.append(levelValue)
                if len(tupleList) != 0:
                    fullTupleList.append(tupleList)
                index+=1
            mi = pd.MultiIndex.from_tuples(fullTupleList,names=levels)
            columnSeriesList = []
            columnTupleList = []
            for column,columnTuple in enumerate(newLevelList):
                columnSeries = pd.Series(rawData.values[:,column+1],index=mi)
                columnSeriesList.append(columnSeries)
                columnTupleList.append(tuple(columnTuple))
            #print(columnTupleList)
            #print(columnSeriesList)
            #print(rawData)
            #print(newLevelList)
            platedf = pd.concat(columnSeriesList,axis=0,keys=columnTupleList,names=dataTypeLevelNames[realDataType])
            dfList.append(platedf)

        idx=pd.IndexSlice 
        fullExperimentDf = pd.concat(dfList)
        #dfl = [fullExperimentDf.xs([12.0],level=['Time']),fullExperimentDf.xs([60.0],level=['Time']),fullExperimentDf.xs([96.0],level=['Time']),fullExperimentDf.xs([156.0],level=['Time'])]
        tempdf = fullExperimentDf.to_frame('temp')
        temp = []
        for row in range(fullExperimentDf.shape[0]):
            name = list(tempdf.iloc[row,:].name)
            if name in temp:
                print(name)
                print(row)
            else:
                temp.append(name)
        #Remove blanks
        for i,level in enumerate(fullExperimentDf.index.names):
            tempLevelValues = pd.unique(fullExperimentDf.index.get_level_values(level))
            if 'Blank' in tempLevelValues.tolist():
                fullExperimentDf = fullExperimentDf.drop('Blank',level=i)
        temp = []
        temp2 = []
        tempdf = fullExperimentDf.to_frame('wat')
        for row in range(fullExperimentDf.shape[0]):
            name = list(tempdf.iloc[row,:].name)
            if name not in temp:
                temp.append(name)
            else:
                temp2.append(name)
        fullExperimentDf = fullExperimentDf.unstack(list(levelLabelDict.keys())[-1])
    
    fullExperimentDf = reorderDfByInputOrder(experimentParameters,fullExperimentDf)
    return fullExperimentDf

def convertDataFramesToExcel(folderName,secondPath,dataType,df):
    writer = pd.ExcelWriter('outputData/excelFiles/excelFile-'+folderName+'-'+dataType+'.xlsx')
    if dataType == 'cyt':
        dfg = pickle.load(open('outputData/pickleFiles/cytokineGFIPickleFile-'+folderName+'.pkl','rb'))
        dfc = pickle.load(open('outputData/pickleFiles/'+dataTypeDataFrameFileNames[dataType]+'-'+folderName+'.pkl','rb'))
        dfg.to_excel(writer,'MFI')
        dfc.to_excel(writer,'Concentration')
    elif dataType == 'killing':
        dfk = pickle.load(open('outputData/pickleFiles/'+dataTypeDataFrameFileNames[dataType]+'-'+folderName+'.pkl','rb'))
        dfk.to_excel(writer,'Killing Index')
    else:
        for statistic in list(pd.unique(df.index.get_level_values('Statistic'))):
            statisticDf = df.xs(statistic,level='Statistic')
            statisticDf.to_excel(writer,statistic)
    writer.save()

def saveFinalDataFrames(folderName,secondPath,experimentNumber,dataType,fullExperimentDf,excel_data):
    fullExperimentDf = fullExperimentDf.astype(float)
    with open('outputData/pickleFiles/'+dataTypeDataFrameFileNames[dataType]+'-'+folderName+'.pkl', "wb") as f:
        pickle.dump(fullExperimentDf, f)
    convertDataFramesToExcel(folderName,secondPath,dataType,fullExperimentDf)
    print(fullExperimentDf)



#!/usr/bin/env python3
import math
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
    
def returnLogYTicksAndLabels(yvals):
    miny = math.floor(yvals.values.min())
    maxy = math.ceil(yvals.values.max())
    allyticks = list(range(miny,maxy+1))
    allyticklabels = []
    for ytick in allyticks:
        allyticklabels.append('$10^{'+str(ytick)+'}$')
    minoryticks = np.log10(list(range(2,10)))
    allminoryticks = []
    for ytick in allyticks[:-1]:
        for minory in minoryticks:
            allminoryticks.append(ytick+minory)
    return allyticks,allyticklabels,allminoryticks

def plot(plottingDf,subsettedDf,kwargs,facetKwargs,auxillaryKwargs,plotOptions):
    if plotOptions['Y']['axisScaling'] == 'Linear':
        errorBar = 'sd'
    else:
        errorBar = 99
    if auxillaryKwargs['subPlotType'] == 'point':
        if auxillaryKwargs['addDistributionPoints']:
            fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'],ci=errorBar,join=False,color='k',capsize=0.05,markers='_',zorder=3,errwidth=1,**plotOptions['X']['figureDimensions'],**auxillaryKwargs['cmap'])
        else:
            fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'],ci=errorBar,join=False,capsize=0.05,errwidth=1,**plotOptions['X']['figureDimensions'],**auxillaryKwargs['cmap'])
    elif auxillaryKwargs['subPlotType'] == 'box':
        fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'])
    elif auxillaryKwargs['subPlotType'] == 'bar':
        fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'],ci=errorBar,alpha=0.8,errwidth=1,capsize=0.05,**plotOptions['X']['figureDimensions'],**auxillaryKwargs['cmap'])
    #violin,swarm,strip
    elif auxillaryKwargs['subPlotType'] == 'violin':
        fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'],alpha=0.8,scale='width',**plotOptions['X']['figureDimensions'],**auxillaryKwargs['cmap'])
    elif  auxillaryKwargs['subPlotType'] in ['swarm','strip']:
        if plotOptions['Y']['axisScaling'] == 'Logarithmic':
            minVal = min(plottingDf[kwargs['y']])
            if minVal <= 0:
                plottingDf[kwargs['y']] = plottingDf[kwargs['y']]+abs(minVal)+1
            plottingDf[kwargs['y']] = np.log10(plottingDf[kwargs['y']])
        fg = sns.catplot(**kwargs,**facetKwargs,data=plottingDf,kind=auxillaryKwargs['subPlotType'],alpha=0.7,edgecolor='black',linewidth=0.3,dodge=True,**plotOptions['X']['figureDimensions'],**auxillaryKwargs['cmap'])
    if auxillaryKwargs['addDistributionPoints']:
        NoneType = type(None)
        secondkwargs = kwargs.copy()
        for key in ['row','col','col_order','row_order','col_wrap']:
            if key in secondkwargs.keys():
                secondkwargs.pop(key,None)
        if auxillaryKwargs['subPlotType'] != 'violin':
            secondkwargs['dodge'] = True
        secondkwargs['edgecolor'] = 'black'
        secondkwargs['linewidth'] = 0.3
        secondkwargs['zorder'] = 1
        swarm = True 
        axisIndex  = 0
        if 'row' in kwargs and 'col' in kwargs:
            for rowVal in pd.unique(plottingDf[kwargs['row']]):
                for colVal in pd.unique(plottingDf[kwargs['col']]):
                    secondPlottingDf = plottingDf[plottingDf[kwargs['row']] == rowVal]
                    secondPlottingDf = secondPlottingDf[secondPlottingDf[kwargs['col']] == colVal]
                    if auxillaryKwargs['subPlotType'] != 'violin' and swarm == False:
                        a = sns.stripplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    else:
                        a = sns.swarmplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    if rowVal != pd.unique(plottingDf[kwargs['row']])[-1]:
                        fg.fig.axes[axisIndex].set_xlabel('')
                    if not isinstance(a.legend_, NoneType):
                        a.legend_.remove()
                    axisIndex+=1
        else:
            if 'row' in kwargs:
                for rowVal in pd.unique(plottingDf[kwargs['row']]):
                    secondPlottingDf = plottingDf[plottingDf[kwargs['row']] == rowVal]
                    if auxillaryKwargs['subPlotType'] != 'violin' and swarm == False:
                        a = sns.stripplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    else:
                        a = sns.swarmplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    if rowVal != pd.unique(plottingDf[kwargs['row']])[-1]:
                        fg.fig.axes[axisIndex].set_xlabel('')
                    axisIndex+=1
                    if not isinstance(a.legend_, NoneType):
                        a.legend_.remove()
            elif 'col' in kwargs:
                for colVal in pd.unique(plottingDf[kwargs['col']]):
                    secondPlottingDf = plottingDf[plottingDf[kwargs['col']] == colVal]
                    if auxillaryKwargs['subPlotType'] != 'violin' and swarm == False:
                        a = sns.stripplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    else:
                        a = sns.swarmplot(**secondkwargs,data=secondPlottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                    if not isinstance(a.legend_, NoneType):
                        a.legend_.remove()
                    #When col wrapping:
                    if 'col_wrap' in kwargs:
                        cw = kwargs['col_wrap']
                        bottomrow = int(len(list(pd.unique(plottingDf[kwargs['col']])))/cw)*cw
                        #Remove all but bottom row x axis labels
                        if axisIndex < bottomrow:
                            fg.fig.axes[axisIndex].set_xlabel('')
                        #Remove all but leftmost y axis labels
                        if axisIndex % cw != 0:
                            fg.fig.axes[axisIndex].set_ylabel('')
                    axisIndex+=1
 
            else:
                if auxillaryKwargs['subPlotType'] != 'violin' and swarm == False:
                    a = sns.stripplot(**secondkwargs,data=plottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                else:
                    a = sns.swarmplot(**secondkwargs,data=plottingDf,ax=fg.fig.axes[axisIndex],**auxillaryKwargs['cmap'])
                if not isinstance(a.legend_, NoneType):
                    a.legend_.remove()
        #if plotOptions['Y']['axisScaling'] == 'Logarithmic':
            #for ax in fg.axes.flat:
                #ax.set_ylim(min(plottingDf[kwargs['y']])+np.log10(0.8),max(plottingDf[kwargs['y']])+np.log10(1.2))
    
    if auxillaryKwargs['subPlotType'] in ['swarm','strip'] and plotOptions['Y']['axisScaling'] == 'Logarithmic':
        allyticks,allyticklabels,allminoryticks = returnLogYTicksAndLabels(plottingDf[kwargs['y']])
        for axis in fg.axes.flat:
            axis.set_ylim(bottom=allyticks[0],top=allyticks[-1])
            axis.set_yticks(allyticks)
            axis.set_yticklabels(allyticklabels)
            axis.yaxis.set_minor_locator(plt.FixedLocator(allminoryticks))
            if plotOptions['Y']['limit'][0] != '' or plotOptions['Y']['limit'][0] != '':
                for axis in fg.axes.flat:
                    newlims = np.log10(list(map(float,plotOptions['Y']['limit']))).tolist()
                    axis.set_ylim(newlims)
    else:
        #X and Y Axis Scaling for 2D plots
        for axis in plotOptions:
            k = len(fg.fig.get_axes())
            if 'Y' in axis:
                if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                    for i in range(k):
                        fg.fig.get_axes()[i].set_yscale('log')
                elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                    for i in range(k):
                        fg.fig.get_axes()[i].set_yscale('symlog',linthreshx=plotOptions[axis]['linThreshold'])
                
                if str(plotOptions[axis]['limit'][0]) != '' or str(plotOptions[axis]['limit'][1]) != '':
                    for i in range(k):
                        if str(plotOptions[axis]['limit'][0]) != '' and str(plotOptions[axis]['limit'][1]) != '':
                            fg.fig.get_axes()[i].set_ylim(bottom=float(plotOptions[axis]['limit'][0]),top=float(plotOptions[axis]['limit'][1]))
                        else:
                            if str(plotOptions[axis]['limit'][0]) != '':
                                fg.fig.get_axes()[i].set_ylim(bottom=float(plotOptions[axis]['limit'][0]))
                            else:
                                fg.fig.get_axes()[i].set_ylim(top=float(plotOptions[axis]['limit'][1]))
            else:
                if plotOptions[axis]['axisScaling'] == 'Logarithmic':
                    for i in range(k):
                        fg.fig.get_axes()[i].set_xscale('log')
                elif plotOptions[axis]['axisScaling'] == 'Biexponential':
                    for i in range(k):
                        fg.fig.get_axes()[i].set_xscale('symlog',linthreshx=plotOptions[axis]['linThreshold']) 
                
                if str(plotOptions[axis]['limit'][0]) != '' or str(plotOptions[axis]['limit'][1]) != '':
                    for i in range(k):
                        if str(plotOptions[axis]['limit'][0]) != '' and str(plotOptions[axis]['limit'][1]) != '':
                            fg.fig.get_axes()[i].set_xlim(bottom=float(plotOptions[axis]['limit'][0]),top=float(plotOptions[axis]['limit'][1]))
                        else:
                            if str(plotOptions[axis]['limit'][0]) != '':
                                fg.fig.get_axes()[i].set_xlim(bottom=float(plotOptions[axis]['limit'][0]))
                            else:
                                fg.fig.get_axes()[i].set_xlim(top=float(plotOptions[axis]['limit'][1]))
    return fg



#!/usr/bin/env python3 
import pandas as pd

def parseCellCSVHeaders(columns,panelData=[]):

#,SingleCells/CD45Neg/TumorCells | Count,SingleCells/CD45Neg/TumorCells | Geometric Mean (Comp-APC-A),SingleCells/CD45Neg/TumorCells | Geometric Mean (Comp-BV650-A),SingleCells/CD45Neg/TumorCells | Geometric Mean (Comp-PE ( 561 )-A),SingleCells/CD45Pos/TCells | Count,SingleCells/CD45Pos/TCells | Geometric Mean (Comp-APC-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-APC-Cy7-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BUV737-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BV421-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BV510-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BV650-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BV711-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-BV786-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-FITC-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-PE-CF594-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-PE-Cy7-A),SingleCells/CD45Pos/TCells | Geometric Mean (Comp-PerCP-Cy5-5-A),SingleCells/CD45Pos/TCells/CD25+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD25+ | Count,SingleCells/CD45Pos/TCells/CD27+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD27+ | Count,SingleCells/CD45Pos/TCells/CD39+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD39+ | Count,SingleCells/CD45Pos/TCells/CD44+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD44+ | Count,SingleCells/CD45Pos/TCells/CD54+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD54+ | Count,SingleCells/CD45Pos/TCells/CD69+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/CD69+ | Count,SingleCells/CD45Pos/TCells/PD1+ | Freq. of Parent (%),SingleCells/CD45Pos/TCells/PD1+ | Count,
    #,Cells/Single Cells/APCs | Geometric Mean (Comp-BV605-A),Cells/Single Cells/APCs | Geometric Mean (Comp-FITC-A),Cells/Single Cells/APCs | Geometric Mean (Comp-PE ( 561 )-A),Cells/Single Cells/APCs | Count,Cells/Single Cells/APCs/CD86+ | Freq. of Parent (%),Cells/Single Cells/APCs/H2Kb+ | Freq. of Parent (%),Cells/Single Cells/APCs/PDL1+ | Freq. of Parent (%),Cells/Single Cells/TCells | Count,Cells/Single Cells/TCells | Geometric Mean (Comp-BUV737-A),Cells/Single Cells/TCells | Geometric Mean (Comp-BV605-A),Cells/Single Cells/TCells | Geometric Mean (Comp-PE-CF594-A),Cells/Single Cells/TCells | Geometric Mean (Comp-PE-Cy7-A),Cells/Single Cells/TCells | Geometric Mean (Comp-PerCP-Cy5-5-A),Cells/Single Cells/TCells/CD27+ | Freq. of Parent (%),Cells/Single Cells/TCells/CD54+ | Freq. of Parent (%),Cells/Single Cells/TCells/CD69+ | Freq. of Parent (%),Cells/Single Cells/TCells/PDL1+ | Freq. of Parent (%),
    newMultiIndexList = []
    for column in columns:
        if 'Unnamed' not in column:
            populationNameVsStatisticSplit = column.split(' | ')
            fullPopulationName = populationNameVsStatisticSplit[0]
            #Statistics can be performed on the whole cell population, in which case the cellType is allEvents
            #MFI and CV need to be specified in terms of a laser channel; count and percent positive do not
            if '/' in fullPopulationName:
                populationDivisionIndices = [i for i,c in enumerate(fullPopulationName) if c=='/']
                #MFI or CV
                #print(populationNameVsStatisticSplit[1])
                if 'Mean' in populationNameVsStatisticSplit[1] or 'Median' in populationNameVsStatisticSplit[1] or 'CV' in populationNameVsStatisticSplit[1]:
                    cellType = fullPopulationName
                    #cellType = fullPopulationName[populationDivisionIndices[-1]+1:]
                    if 'Comp-' in populationNameVsStatisticSplit[1]:
                        statisticVsChannelSplit = populationNameVsStatisticSplit[1].split(' (Comp-')
                    else:
                        statisticVsChannelSplit = populationNameVsStatisticSplit[1].split(' (')
                    statistic = statisticVsChannelSplit[0]
                    #print(statistic)
                    if 'Mean' in statistic:
                        statisticName = 'MFI'
                    elif 'Median' in statistic:
                        statisticName = 'MedianFI'
                    else:
                        statisticName = 'CV'
                    
                    #Can have IRF4+,TCells,CD45RB+CD25-; first case: cellType = first case:cellType=previousPop,marker=currentPop[:-1],stat=Positive/Negative MFI; 
                    #second case: cellType = currentPop,marker=NotApplicable,statistic=MFI; third case: cellType = currentPop,marker=NotApplicable,statistic=MFI
                    numPositive = fullPopulationName[populationDivisionIndices[-1]+1:].count('+')
                    numNegative = fullPopulationName[populationDivisionIndices[-1]+1:].count('-')
                    channel = statisticVsChannelSplit[1][:-1]
                    if '-A' in channel:
                        if '::' not in statisticVsChannelSplit[1]:
                            panelIndex = list(panelData['FCSDetectorName']).index(channel)
                            marker = panelData['Marker'][panelIndex]
                        else:
                            marker = statisticVsChannelSplit[1].split(' :: ')[-1][:-1] 
                    else:
                        if '::' not in statisticVsChannelSplit[1]:
                            marker = channel 
                        else:
                            marker = statisticVsChannelSplit[1].split(' :: ')[-1][:-1] 
                    #if numPositive+numNegative == 0, just use the last gate as the population name:
                    if numPositive+numNegative == 0:
                        cellType = fullPopulationName[populationDivisionIndices[-1]+1:]
                        statistic = statisticName
                    #If more than 1 +/- signs, include full population gating hiearchy as cell population name
                    elif numPositive+numNegative > 1:
                        cellType = fullPopulationName
                        statistic = statisticName
                    #If just a single + or - sign, extract marker name out of last gate
                    else:
                        positivePop = fullPopulationName[populationDivisionIndices[-1]+1:-1]
                        if positivePop == marker:
                            if len(populationDivisionIndices) > 1:
                                cellType = fullPopulationName[populationDivisionIndices[-2]+1:populationDivisionIndices[-1]]
                            else:
                                cellType = fullPopulationName[:populationDivisionIndices[-1]]
                        else:
                            cellType = fullPopulationName
                        if numNegative == 0: 
                            if cellType.split('/')[-1][:-1] == marker:
                                statistic = 'Positive '+statisticName
                            else:
                                statistic = statisticName 
                        else:
                            if cellType.split('/')[-1][:-1] == marker:
                                statistic = 'Negative '+statisticName
                            else:
                                statistic = statisticName
                #% of parent and count
                else:
                    numPositive = fullPopulationName[populationDivisionIndices[-1]+1:].count('+')
                    numNegative = fullPopulationName[populationDivisionIndices[-1]+1:].count('-')
                    if 'Freq' in populationNameVsStatisticSplit[1]:
                        if numPositive+numNegative == 0:
                            marker = 'NotApplicable'
                            statistic ='%'
                            #cellType = fullPopulationName[populationDivisionIndices[-1]+1:]
                            cellType = fullPopulationName
                        elif numPositive+numNegative > 1:
                            marker = 'NotApplicable'
                            statistic = '%'
                            cellType = fullPopulationName
                        else:
                            if numNegative == 0:
                                statistic = '% Positive'
                            else:
                                statistic = '% Negative'
                            marker = fullPopulationName[populationDivisionIndices[-1]+1:len(fullPopulationName)-1]
                            positivePop = fullPopulationName[populationDivisionIndices[-1]+1:-1]
                            if positivePop == marker:
                                if len(populationDivisionIndices) > 1:
                                    cellType = fullPopulationName[populationDivisionIndices[-2]+1:populationDivisionIndices[-1]]
                                else:
                                    cellType = fullPopulationName[:populationDivisionIndices[-1]]
                            else:
                                cellType = fullPopulationName
                    else:
                        marker = 'NotApplicable'
                        statistic = 'Count'
                        if numPositive+numNegative == 0:
                            cellType = fullPopulationName[populationDivisionIndices[-1]+1:] 
                        else:
                            cellType = fullPopulationName
            else:
                #Statistics can be performed on the whole cell population, in which case the cellType is allEvents
                #DAPI+ | Freq. of Parent (%)
                #Positive cell percentage statistics do not have channel names, so treat differently
                #MFI or CV
                if len(populationNameVsStatisticSplit) == 1:
                    cellType = 'allEvents'
                    populationNameVsStatisticSplit = [' ',populationNameVsStatisticSplit[0]]
                else:
                    cellType = populationNameVsStatisticSplit[0]
                if 'Mean' in populationNameVsStatisticSplit[1] or 'Median' in populationNameVsStatisticSplit[1] or 'CV' in populationNameVsStatisticSplit[1]:
                    if 'Comp-' in populationNameVsStatisticSplit[1]:
                        statisticVsChannelSplit = populationNameVsStatisticSplit[1].split(' (Comp-')
                    else:
                        statisticVsChannelSplit = populationNameVsStatisticSplit[1].split(' (')
                    statistic = statisticVsChannelSplit[0]
                    if 'Mean' in statistic:
                        statistic = 'MFI'
                    elif 'Median' in statistic:
                        statistic = 'MedianFI'
                    else:
                        statistic = 'CV'
                    channel = statisticVsChannelSplit[1][:-1]
                    marker = channel
                    #panelIndex = list(panelData['FCSDetectorName']).index(channel)
                    #marker = panelData['Marker'][panelIndex]
                #% of parent and count
                else:
                    marker = 'NotApplicable'
                    if 'Freq' in populationNameVsStatisticSplit[1]:
                        statistic = '% Positive'
                    else:
                        statistic = 'Count'
            newMultiIndexList.append([cellType,marker,statistic])
    
    commonBranchesIndices = []
    commonBranches = []
    for multiIndexList in newMultiIndexList:
        fullPopulationName = multiIndexList[0]
        populationDivisionIndices = [i for i,c in enumerate(fullPopulationName) if c=='/']
        commonBranches.append(fullPopulationName.split('/'))
        commonBranchesIndices.append(populationDivisionIndices)
    commonIndex = 0
    masterBranchList = []
    for branchlist in commonBranches:
        for branch in branchlist:
            masterBranchList.append(branch)
    uniqueBranches = list(pd.unique(masterBranchList))
    commonToAllStatistics = []
    for uniqueBranch in uniqueBranches:
        isCommonToAllStatistics = True
        for branchlist in commonBranches:
            if uniqueBranch not in branchlist:
                isCommonToAllStatistics = False
        if isCommonToAllStatistics:
            commonToAllStatistics.append(uniqueBranch)
    uncommonStatisticsList = []
    for commonBranch in commonBranches:
        uncommonStatistics = []
        for branch in commonBranch:
            if branch not in commonToAllStatistics:
                uncommonStatistics.append(branch)
        uncommonStatisticsList.append('/'.join(uncommonStatistics))
    for uncommonStatistic,i in zip(uncommonStatisticsList,range(len(uncommonStatisticsList))):
        newMultiIndexList[i] = [uncommonStatistic]+newMultiIndexList[i][1:]
    return newMultiIndexList

def createCellDataFrame(folderName,finalDataFrame,concUnitPrefix):
    return finalData



#!/usr/bin/env python3 
import pickle,math,matplotlib,re
from matplotlib import pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
import tkinter as tk
import tkinter.ttk
from collections import OrderedDict
from operator import itemgetter
from ..dataprocessing.miscFunctions import sortSINumerically,setMaxWidth,returnTicks,get_cluster_centroids

def createParameterSelectionRadiobuttons(radiobuttonWindow,parameterList,parameterValueDict):
    radiobuttonList = []
    radiobuttonVarsDict = {}
    for i,parameter in enumerate(parameterList):
        tk.Label(radiobuttonWindow,text=parameter).grid(row=0,column=i,sticky=tk.W)
        parameterValues = parameterValueDict[parameter]
        parameterVar = tk.StringVar()
        for j in range(len(parameterValues)):
            parameterValue = parameterValues[j]
            radiobutton = tk.Radiobutton(radiobuttonWindow,text=parameterValue,value=parameterValue,variable=parameterVar)
            radiobutton.grid(row=j+1,column=i,sticky=tk.W)
        parameterVar.set(parameterValues[0])
        radiobuttonList.append(radiobutton)
        radiobuttonVarsDict[parameter] = parameterVar
    return radiobuttonList,radiobuttonVarsDict

def getRadiobuttonValues(radiobuttonVarsDict):
    parameterDict = {}
    for parameter in radiobuttonVarsDict:
        parameterDict[parameter] = radiobuttonVarsDict[parameter].get()
    return parameterDict

def createParameterAdjustmentSliders(sliderWindow,parameterList,parameterBoundsDict):
    sliderList = []
    for i,parameter in enumerate(parameterList):
        tk.Label(sliderWindow,text=parameter).grid(row=0,column=i)
        parameterBounds = parameterBoundsDict[parameter]
        slider = tk.Scale(sliderWindow, from_=parameterBounds[0], to=parameterBounds[1],resolution=parameterBounds[2])
        slider.grid(row=1,column=i)
        slider.set(parameterBounds[3])
        sliderList.append(slider)
    return sliderList
    
def getSliderValues(sliders,parameterList,mutuallyExclusiveParameterList = []):
    parametersForSliderFunction = {}
    for slider,parameter in zip(sliders,parameterList):
        parametersForSliderFunction[parameter] = slider.get()
    return parametersForSliderFunction

def createParameterSelectionDropdowns(dropdownWindow,parameterList,parameterValueDict,defaultParameterValueDict):
    dropdownList = []
    dropdownVarsDict = {}
    for i,parameter in enumerate(parameterList):
        parameterValues = parameterValueDict[parameter]
        tk.Label(dropdownWindow,text=parameter+': ').grid(row=i,column=0)
        parameterVar = tk.StringVar()
        parameterMenu = tk.OptionMenu(dropdownWindow,parameterVar,*parameterValues)
        parameterMenu.grid(row=i,column=1)
        parameterVar.set(defaultParameterValueDict[parameter])
        setMaxWidth(parameterValues,parameterMenu)
        dropdownList.append(parameterMenu)
        dropdownVarsDict[parameter] = parameterVar
    return dropdownList,dropdownVarsDict

def createParameterSelectionDropdownsWithIndividualLevels(dropdownWindow,parameterList,parameterValueDict,defaultParameterValueDict,plottingDf,experimentParameters):
    #Construct level value dictionary to use with level subsetting dropdowns 
    levelValueDict = {}
    maxLevelLen = 0
    maxLevelValueLen = 0
    for level in parameterValueDict['hue']:
        #Changed from allLevelValues to levelLabelDict
        if level in list(experimentParameters['levelLabelDict'].keys())+['CellType','Cluster']:
            if level in list(experimentParameters['levelLabelDict'].keys()):
                individualLevelValues = experimentParameters['levelLabelDict'][level]
            elif level == 'Cluster':
                individualLevelValues = list(map(str,sorted(list(map(int,list(pd.unique(plottingDf['Cluster'])))))))
            else:
                individualLevelValues = list(pd.unique(plottingDf[level]))
            levelValueDict[level] = ['All']+individualLevelValues
            if type(level) == str:
                if len(level) > maxLevelLen:
                    maxLevelLen = len(level)
            for levelVal in individualLevelValues:
                if type(levelVal) == str:
                    if len(levelVal) > maxLevelValueLen:
                        maxLevelValueLen = len(levelVal)

    def getUpdateData(event):
        dropdownWindow.levelValueCombo['values'] = levelValueDict[dropdownWindow.levelCombo.get()]
        dropdownWindow.levelValueCombo.set(dropdownWindow.levelValueCombo['values'][0])
    
    #Construct plotting selection dropdowns
    dropdownList = []
    dropdownVarsDict = {}
    for i,parameter in enumerate(parameterList):
        parameterValues = parameterValueDict[parameter]
        tk.Label(dropdownWindow,text=parameter+': ').grid(row=i,column=0)
        parameterVar = tk.StringVar()
        parameterMenu = tk.OptionMenu(dropdownWindow,parameterVar,*parameterValues)
        parameterMenu.grid(row=i,column=1)
        parameterVar.set(defaultParameterValueDict[parameter])
        setMaxWidth(parameterValues,parameterMenu)
        dropdownList.append(parameterMenu)
        dropdownVarsDict[parameter] = parameterVar
    
    defaultVal = defaultParameterValueDict['hue'] 
    defaultLength = 10
    
    #Construct level and level value subsetting dropdowns =
    tk.Label(dropdownWindow,text='Level: ').grid(row=0,column=2)
    dropdownWindow.levelCombo = tkinter.ttk.Combobox(dropdownWindow,values = list(levelValueDict.keys()))
    dropdownWindow.levelCombo['width'] = max([defaultLength,maxLevelLen])
    dropdownWindow.levelCombo.bind('<<ComboboxSelected>>', getUpdateData)
    dropdownWindow.levelCombo.grid(row = 0,column = 3,sticky=tk.W)
    dropdownWindow.levelCombo.set(defaultVal)

    tk.Label(dropdownWindow,text='Level Value: ').grid(row=1,column=2)
    dropdownWindow.levelValueCombo = tkinter.ttk.Combobox(dropdownWindow,state='readonly')
    dropdownWindow.levelValueCombo['width'] = max([defaultLength,maxLevelValueLen])
    dropdownWindow.levelValueCombo.grid(row = 1,column = 3,sticky=tk.W)
    dropdownWindow.levelValueCombo['values'] = levelValueDict[defaultVal] 
    dropdownWindow.levelValueCombo.set('All')
    
    return dropdownList,dropdownVarsDict,dropdownWindow.levelCombo,dropdownWindow.levelValueCombo

def getDropdownValues(dropdownVarsDict):
    parametersForDropdowns = {}
    for parameter in dropdownVarsDict:
        parametersForDropdowns[parameter] = dropdownVarsDict[parameter].get()
    return parametersForDropdowns

def fixDuckTyping(plottingDf,kwargs):
    #Fix duck typing issue with replots: https://github.com/mwaskom/seaborn/issues/1653
    if 'hue' in kwargs and isinstance(plottingDf[kwargs['hue']][0],str):
        if plottingDf[kwargs['hue']][0].isnumeric():
            plottingDf[kwargs['hue']] = ["$%s$" % x for x in plottingDf[kwargs['hue']]]
    if 'size' in kwargs and isinstance(plottingDf[kwargs['size']][0],str):
        if plottingDf[kwargs['size']][0].isnumeric():
            plottingDf[kwargs['size']] = ["$%s$" % x for x in plottingDf[kwargs['size']]]
    return plottingDf

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def returnOrderedClusters(clusterList):
    numericClusters = []
    clusterDict = {}
    for cluster in clusterList:
        numeric = re.findall(r'\d+', str(cluster))
        clusterDict[numeric] = cluster
        numericClusters.append(numeric)
    orderedNumericClusterList = sorted(numericClusters)
    orderedClusters = []
    for orderedCluster in orderedNumericClusterList:
        orderedClusters.append(str(clusterDict[orderedCluster]))
    return orderedClusters

def returnOriginalOrders(trueLabelDict,plottingDf,kwargs,dimensionality):
    if dimensionality == '2d':
        newkwargs = kwargs.copy()
        a = newkwargs.pop('x')
        b = newkwargs.pop('y')
    else:
        newkwargs = kwargs.copy()
    orderDict = {}
    for kwarg in newkwargs:
        if newkwargs[kwarg] == 'Cluster' and '$' not in str(plottingDf[newkwargs[kwarg]][0]):
            orderedValues = list(map(str,sorted(list(map(int,list(pd.unique(plottingDf['Cluster'])))))))
        else:
            if newkwargs[kwarg] in trueLabelDict.keys():
                originalValues = trueLabelDict[newkwargs[kwarg]]
                newValues = pd.unique(plottingDf[newkwargs[kwarg]])
                orderedValues = []
                for originalValue in originalValues:
                    if originalValue in newValues:
                        orderedValues.append(originalValue)
            else:
                if len(pd.unique(plottingDf[newkwargs[kwarg]])) == 1:
                    orderedValues = list(pd.unique(plottingDf[newkwargs[kwarg]]))
                else:
                    orderedValues = list(pd.unique(plottingDf[newkwargs[kwarg]]))
        if newkwargs[kwarg] != 'None':
            if kwarg == 'x':  
                orderDict['order'] = orderedValues
            else:
                orderDict[kwarg+'_order'] = orderedValues
    return orderDict

def addLogicleAxes(axis,ticks):
    #Add appropriate xtick values (also ytick values if kde) for each axis in figure
    for ax in ticks:
        tickValues,tickLabels = returnTicks(ticks[ax])
        if ax == 'x':
            axis.set_xticks(tickValues)
            axis.set_xticklabels(tickLabels)
        else:
            axis.set_yticks(tickValues)
            axis.set_yticklabels(tickLabels)
         
def addCountYAxis(axis,subplotValuesList):

    #Add in correct y axis labels
    maxCounts = []
    for subplotValues in subplotValuesList:
        #Make sure bins of histogram are over the correct range by appending the appropriate extrema
        newvals = np.append(subplotValues,[[0,1023]])
        hist,_ = np.histogram(newvals, bins=256)
        #remove appended extrema
        hist[0]-=1
        hist[-1]-=1
        maxCount = max(hist)
        maxCounts.append(maxCount)
    trueMax = max(maxCounts)
    oldylabels = axis.get_yticks().tolist()
    oldmax = oldylabels[-1]
    i=0
    minticknum = 5
    factor = 10
    keepIncreasing = True
    while keepIncreasing:
        tickspaces = [1*factor,2*factor,2.5*factor,5*factor,10*factor]
        for j,tickspace in enumerate(tickspaces):
            numticks = int(trueMax/tickspace)
            #uncomment if you want the min tick number to be "minticknumber"
            #if numticks <= minticknum:
            #uncomment if you want the max tick number to be "minticknumber"
            if numticks <= minticknum:
                if j == 0:
                    finalTickLength = tickspaces[0]
                else:
                    #uncomment if you want the max tick number to be "minticknumber"
                    finalTickLength = tickspaces[j]
                    #uncomment if you want the min tick number to be "minticknumber"
                    #finalTickLength = tickspaces[j-1]
                keepIncreasing = False
                break
        factor*=10
    
    if maxCounts[i] > 0:
        finalNumticks = int(maxCounts[i]/finalTickLength)+2
        if finalNumticks <= 2:
            finalNumticks = 3
        oldTickLength = oldmax/(finalNumticks-1)
        newyticklabels = []
        newyticks = []
        for i in range(finalNumticks):
            newyticks.append(i*oldTickLength)
            newyticklabels.append(int(i*finalTickLength))
        axis.set_yticks(newyticks)
        axis.set_yticklabels(newyticklabels)

def updateDropdownControlledCompositionPlot(frameCanvas,plotAxis,plottingDf,trueLabelDict,levelVars,legendoffset=1.7):
    plotAxis.clear()
    if not isinstance(plottingDf,list):
        parameters = [levelVars['x'].get(),levelVars['y'].get(),levelVars['hue'].get()]
        for i,parameter in enumerate(parameters):
            if parameter == 'Group':
                parameters[i] = 'Cluster'
        parameters2 = ['x','y','hue']
        newkwargs = {}
        for parameter,parameter2 in zip(parameters,parameters2):
            if parameter != 'None':
                newkwargs[parameter2] = parameter
        #scatter plot (2d)
        if 'x' in newkwargs and 'y' in newkwargs and newkwargs['y'] not in ['frequency','log-frequency','percent']:
            orderedClusters = list(map(str,sorted(list(map(int,list(pd.unique(plottingDf['Cluster'])))))))
            modifiedNewKwargs = newkwargs.copy()
            featureX = modifiedNewKwargs.pop('x')
            featureY = modifiedNewKwargs.pop('y')
            valuesX = list(plottingDf[plottingDf['Feature'] == featureX]['Metric'])
            valuesY = list(plottingDf[plottingDf['Feature'] == featureY]['Metric'])
            featureHueBool = False
            paletteKwargs = {}
            if 'hue' in newkwargs:
                if newkwargs['hue'] in list(pd.unique(plottingDf['Feature'])):
                    featureHue = modifiedNewKwargs.pop('hue')
                    valuesHue = list(plottingDf[plottingDf['Feature'] == featureHue]['Metric'])
                    featureHueBool = True
                    palette = 'coolwarm'
                else:
                    if newkwargs['hue'] == 'Time':
                        palette = 'coolwarm'
                    else:
                        palette = sns.color_palette(sns.color_palette(),len(pd.unique(plottingDf[newkwargs['hue']])))
                paletteKwargs['palette'] = palette
            orderDict = returnOriginalOrders(trueLabelDict,plottingDf,modifiedNewKwargs,'1.5d')
            plottingDf = plottingDf[plottingDf['Feature'] == plottingDf['Feature'][0]]
            plottingDf[featureX] = valuesX
            plottingDf[featureY] = valuesY
            if featureHueBool:
                plottingDf[featureHue] = valuesHue
                hueNormKwargs = {'hue_norm':(0,1000)}
            else:
                hueNormKwargs = {}
            hue_orderDict = {}
            if newkwargs['hue'] == 'Cluster':
                hue_orderDict['hue_order'] = orderedClusters 
            else:
                if 'hue_order' in orderDict:
                    hue_orderDict['hue_order'] = orderDict['hue_order'] 
            g3 = sns.scatterplot(data=plottingDf,**newkwargs,**hue_orderDict,**paletteKwargs,s=5,alpha=0.7,**hueNormKwargs)
            leg = g3.legend(loc='center right', bbox_to_anchor=(legendoffset, 0.5), ncol=1,framealpha=0)
            
            if featureHueBool:
                a,b = returnTicks([-1000,100,10000,100000])
                for t, l in zip(leg.texts[1:],(b)):
                    t.set_text(l)
            
            if max(plottingDf['Metric']) > 100:
                tickDict = {}
                ticks = [-1000,100,1000,10000,100000]
                tickDict['x'] = ticks
                tickDict['y'] = ticks
                addLogicleAxes(plotAxis,tickDict)
        
            #Rotate index labels if x not cluster
            #If feature plot on x axis, change x axis name
             
        #barplot (1.5d) or kde (1d)
        else:
            if newkwargs['y'] in ['frequency','log-frequency','percent']:
                yaxisFeature = newkwargs.pop('y')
            else:
                yaxisFeature = newkwargs['y']
            orderedClusters = list(map(str,sorted(list(map(int,list(pd.unique(plottingDf['Cluster'])))))))
            if newkwargs['x'] in list(pd.unique(plottingDf['Feature'])):
                modifiedNewKwargs = newkwargs.copy()
                feature = modifiedNewKwargs.pop('x')
                featureBool = True
                newkwargs['x'] = 'Metric'
            else:
                modifiedNewKwargs = newkwargs.copy()
                featureBool = False
            orderDict = returnOriginalOrders(trueLabelDict,plottingDf,modifiedNewKwargs,'1.5d')
            if not featureBool:
                palette = sns.color_palette(sns.color_palette(),len(pd.unique(plottingDf['Cluster'])))
            else:
                if 'hue' in newkwargs:
                    palette = sns.color_palette(sns.color_palette(),len(pd.unique(plottingDf[newkwargs['hue']])))
                else:
                    palette = sns.color_palette(sns.color_palette(),1)
            kdePlotBool = False
            if yaxisFeature in ['frequency','log-frequency']:
                if is_number(plottingDf[newkwargs['x']][0]) and len(pd.unique(plottingDf[newkwargs['x']])) > 4 and newkwargs['x'] != 'Cluster':
                    if not featureBool:
                        #"Unstack" dataframe
                        plottingDf = plottingDf[plottingDf['Feature'] == plottingDf['Feature'][0]]
                        for i,cluster in enumerate(orderedClusters):
                            g3 = sns.kdeplot(plottingDf[plottingDf['Cluster'] == cluster][newkwargs['x']],color=palette[i],ax=plotAxis,shade=True,label=cluster)
                    else:
                        plottingDf = plottingDf[plottingDf['Feature'] == feature]
                        if newkwargs['hue'] == 'Cluster':
                            hue_order = orderedClusters 
                        else:
                            hue_order = orderDict['hue_order'] 
                        for i,cluster in enumerate(hue_order):
                            g3 = sns.kdeplot(plottingDf[plottingDf[newkwargs['hue']] == cluster][newkwargs['x']],color=palette[i],ax=plotAxis,shade=True,label=cluster)
                        if max(plottingDf['Metric']) > 100:
                            tickDict = {}
                            ticks = [-1000,100,1000,10000,100000]
                            tickDict['x'] = ticks
                            addLogicleAxes(plotAxis,tickDict)
                            if 'hue' in newkwargs:
                                subplotValues = []
                                for hueval in pd.unique(plottingDf[newkwargs['hue']]):
                                    subplotValues.append(plottingDf[plottingDf[newkwargs['hue']] == hueval]['Metric'])
                            else:
                                subplotValues = [plottingDf['Metric']]
                            addCountYAxis(plotAxis,subplotValues)
            
                    kdePlotBool = True
                else:
                    #"Unstack" dataframe
                    plottingDf = plottingDf[plottingDf['Feature'] == plottingDf['Feature'][0]]
                    g3 = sns.countplot(data=plottingDf,ax=plotAxis,**newkwargs,**orderDict,edgecolor='black',linewidth=1)
                    #Log frequency if needed
                    #Log frequency if needed
                    if yaxisFeature == 'log-frequency':
                        plotAxis.set_yscale('log')
            else: 
                #"Unstack" dataframe
                plottingDf = plottingDf[plottingDf['Feature'] == plottingDf['Feature'][0]]
                if 'hue' in newkwargs.keys():
                    x,y = newkwargs['x'],newkwargs['hue']
                    plottingDf = plottingDf.groupby(x)[y].value_counts(normalize=True).mul(100).rename('percent').reset_index()
                else:
                    plottingDf = plottingDf[newkwargs['x']].value_counts(normalize=True).mul(100)
                    plottingDf.index.names = [newkwargs['x']]
                    plottingDf = plottingDf.to_frame('percent').reset_index()
                newkwargs['y'] = yaxisFeature 
                g3 = sns.barplot(data=plottingDf,ax=plotAxis,**newkwargs,**orderDict,edgecolor='black',linewidth=1)
                #Log frequency if needed
                if yaxisFeature == 'log-frequency':
                    plotAxis.set_yscale('log')
        
            #Rotate index labels if x not cluster
            if newkwargs['x'] != 'Cluster':
                #if not kdeplot, rotate index labels
                if not kdePlotBool:
                    plt.setp(plotAxis.xaxis.get_majorticklabels(), rotation=45)
                #If feature plot on x axis, change x axis name
                if featureBool:
                    plotAxis.set(xlabel=feature)
            else:
                #If count plot, color
                if not kdePlotBool:
                    for i,cluster in enumerate(orderedClusters):
                        plotAxis.get_xticklabels()[i].set_color(palette[i])
                plotAxis.set(xlabel='Group')
            #Correct legend for kde plot
            if kdePlotBool:
                if newkwargs['hue'] == 'Cluster':
                    newkwargs['hue'] = 'Group'
                leg = g3.legend(loc='center right', bbox_to_anchor=(legendoffset, 0.5), ncol=1,framealpha=0,title=newkwargs['hue'])
            else:
                leg = g3.legend(loc='center right', bbox_to_anchor=(legendoffset, 0.5), ncol=1,framealpha=0)

    frameCanvas.draw()

#Assign level with largest num unique values to hue, 2nd largest to style, time to size to start
def getDefaultKwargs(df):
    kwargs = {}
    responseColumns = []
    numUniqueElements = []
    tempDict = {}
    for column in df.columns:
        if 'Dimension' not in column and 'Time' not in column and 'Replicate' not in column:
            responseColumns.append(column)
            numUniqueElements.append(len(list(pd.unique(df[column]))))
            tempDict[column] = len(list(pd.unique(df[column])))
    columnsLeftToAssign = responseColumns.copy()
    sortedNumUniqueElements = sorted(numUniqueElements)[::-1] 
    numUniqueElements2 = numUniqueElements.copy()
    sortedNumUniqueElements2 = sortedNumUniqueElements.copy()
    
    sortedTempDict = OrderedDict(sorted(tempDict.items(), key=itemgetter(1),reverse=True))
    #sortedTempDict = sorted(tempDict, key=tempDict.get)

    #Assign hue variable
    maxUniqueElementsColumn = responseColumns[numUniqueElements.index(sortedNumUniqueElements[0])]
    kwargs['hue'] = maxUniqueElementsColumn
    
    columnsLeftToAssign.remove(maxUniqueElementsColumn)
    numUniqueElements2.remove(sortedNumUniqueElements2[0])
    sortedNumUniqueElements2 = sortedNumUniqueElements2[1:]
    
    if len(sortedNumUniqueElements) > 1:
        #Assign style variable
        if sortedNumUniqueElements[1] < 7:
            secondMaxUniqueElementsColumn = list(sortedTempDict.keys())[1]
            #secondMaxUniqueElementsColumn = responseColumns[numUniqueElements.index(sortedNumUniqueElements[1])]
            kwargs['style'] = secondMaxUniqueElementsColumn
            columnsLeftToAssign.remove(secondMaxUniqueElementsColumn)
            numUniqueElements2.remove(sortedNumUniqueElements2[0])
            sortedNumUniqueElements2 = sortedNumUniqueElements2[1:]
    
    if len(sortedNumUniqueElements) > 2:
        #Assign size variable
        if len(columnsLeftToAssign) > 0:
            if sortedNumUniqueElements2[0] > 1:
                thirdMaxUniqueElementsColumn = columnsLeftToAssign[numUniqueElements2.index(sortedNumUniqueElements2[0])]
                kwargs['size'] = thirdMaxUniqueElementsColumn
            else:
                if 'Time' in df.columns and len(pd.unique(df['Time'])) > 1:
                    kwargs['size'] = 'Time'
        else:
            if 'Time' in df.columns and len(pd.unique(df['Time'])) > 1:
                kwargs['size'] = 'Time'
    
    defaultDict = {'hue':'None','style':'None','size':'None'}
    defaultplotkwargs = kwargs.copy()
    if 'Event' not in df.columns and 'event' not in df.columns:
        if 'hue' in defaultplotkwargs.keys():
            defaultDict['hue'] = defaultplotkwargs['hue']
        if 'style' in defaultplotkwargs.keys():
            defaultDict['style'] = defaultplotkwargs['style']
        if 'size' in defaultplotkwargs.keys():
            defaultDict['size'] = defaultplotkwargs['size']
    return kwargs,defaultDict

def updateDropdownControlledPlot(frameCanvas,plotAxis,plottingDf,levelVars,xColumn,yColumn,alpha=0.3,legendoffset=-0.1,trueLabelDict = [],levelDropdown = [],levelValueDropdown = [],axisLimits=[]):
    plotAxis.clear()

    parameters = [levelVars['hue'].get(),levelVars['style'].get(),levelVars['size'].get()]
    parameters2 = ['hue','style','size']
    newkwargs = {'x':xColumn,'y':yColumn}
    numLegendElements = 0
    for parameter,parameter2 in zip(parameters,parameters2):
        if parameter != 'None':
            newkwargs[parameter2] = parameter
            val = plottingDf[parameter][0]
            if isinstance(val,(int,float,np.integer)):
                numLegendElements+=4
            else:
                numLegendElements+=len(pd.unique(plottingDf[parameter]))
    
    features = list(plottingDf.columns)[list(plottingDf.columns).index('Dimension 2')+1:-1]
    if 'hue' in newkwargs:
        if newkwargs['hue'] in features and max(plottingDf[features[0]]) > 100:
            featureHueBool = True
        else:
            featureHueBool = False 
    else:
        featureHueBool = False 
    if not isinstance(trueLabelDict,list):
        orderDict = returnOriginalOrders(trueLabelDict,plottingDf,newkwargs,'2d')
    else:
        orderDict = {}
    if 'hue' in newkwargs.keys():
        if not isinstance(plottingDf[newkwargs['hue']][0],str):
            palette = 'coolwarm'
            newkwargs['palette'] = palette
        else:
            palette = sns.color_palette(sns.color_palette(),len(pd.unique(plottingDf[newkwargs['hue']]))) 
            newkwargs['palette'] = palette
    if 'hue' in newkwargs.keys():
        if newkwargs['hue'] == 'Cluster':
            clusters = list(pd.unique(plottingDf['Cluster']))
            tempDict = {}
            for i in range(len(clusters)):
                cluster = clusters[i]
                numeric = re.findall(r'\d+', str(cluster))
                tempDict[int(numeric[0])] = cluster
            sortedTempDict = OrderedDict(sorted(tempDict.items()))
            orderDict['hue_order'] = list(sortedTempDict.values())
    if featureHueBool:
        hueNormKwargs = {'hue_norm':(0,1000)}
    else:
        hueNormKwargs = {} 
    if type(levelValueDropdown) != list:
        individualLevelValue = levelValueDropdown.get()
        if individualLevelValue != 'All' and individualLevelValue != '':
            subsetValueBool = True
            truePlottingDf = plottingDf.copy()[plottingDf[levelDropdown.get()] == individualLevelValue]
        else:
            subsetValueBool = False
            truePlottingDf = plottingDf.copy()
    else:
        truePlottingDf = plottingDf.copy()
        subsetValueBool = False
        individualLevelValue = 'All'
    if 'Event' in plottingDf.columns or 'event' in plottingDf.columns:
        g3 = sns.scatterplot(data=truePlottingDf,ax=plotAxis,alpha=0.7,s=3,**newkwargs,**orderDict,**hueNormKwargs)
        if subsetValueBool:
            plotAxis.set_xlim(axisLimits[0])
            plotAxis.set_ylim(axisLimits[1])
    else:
        g3 = sns.scatterplot(data=plottingDf,ax=plotAxis,alpha=0.7,**newkwargs,**orderDict,**hueNormKwargs)
    if 'hue' in newkwargs.keys():
        if newkwargs['hue'] == 'Cluster':
            clusterCentroids = get_cluster_centroids(truePlottingDf)
            for i in range(len(clusterCentroids)):
                g3.annotate(clusterCentroids[i][0],xy=clusterCentroids[i][1])
    legendSpillover = 15
    leg = g3.legend(loc='center right', bbox_to_anchor=(legendoffset, 0.5), ncol=math.ceil(numLegendElements/legendSpillover),framealpha=0)
    if featureHueBool:
        a,b = returnTicks([-1000,100,10000,100000])
        for t, l in zip(leg.texts[1:],(b)):
            t.set_text(l)
    
    frameCanvas.draw()

